@@ -22,7 +22,8 @@ THE SOFTWARE.
 package cluster
 
 import (
-	log "github.com/sirupsen/logrus"
+	l "github.com/rancher/k3d/v5/pkg/logger"
+
 	"github.com/spf13/cobra"
 )
 
@@ -36,8 +37,8 @@ func NewCmdCluster() *cobra.Command {
 		Long:  `Manage cluster(s)`,
 		Run: func(cmd *cobra.Command, args []string) {
 			if err := cmd.Help(); err != nil {
-				log.Errorln("Couldn't get help text")
-				log.Fatalln(err)
+				l.Log().Errorln("Couldn't get help text")
+				l.Log().Fatalln(err)
 			}
 		},
 	}
@@ -48,6 +49,7 @@ func NewCmdCluster() *cobra.Command {
 	cmd.AddCommand(NewCmdClusterStop())
 	cmd.AddCommand(NewCmdClusterDelete())
 	cmd.AddCommand(NewCmdClusterList())
+	cmd.AddCommand(NewCmdClusterEdit())
 
 	// add flags
 
