@@ -32,16 +32,18 @@ import (
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	"github.com/rancher/k3d/v4/cmd/cluster"
-	cfg "github.com/rancher/k3d/v4/cmd/config"
-	"github.com/rancher/k3d/v4/cmd/image"
-	"github.com/rancher/k3d/v4/cmd/kubeconfig"
-	"github.com/rancher/k3d/v4/cmd/node"
-	"github.com/rancher/k3d/v4/cmd/registry"
-	cliutil "github.com/rancher/k3d/v4/cmd/util"
-	"github.com/rancher/k3d/v4/pkg/runtimes"
-	"github.com/rancher/k3d/v4/version"
-	log "github.com/sirupsen/logrus"
+	"github.com/rancher/k3d/v5/cmd/cluster"
+	cfg "github.com/rancher/k3d/v5/cmd/config"
+	"github.com/rancher/k3d/v5/cmd/debug"
+	"github.com/rancher/k3d/v5/cmd/image"
+	"github.com/rancher/k3d/v5/cmd/kubeconfig"
+	"github.com/rancher/k3d/v5/cmd/node"
+	"github.com/rancher/k3d/v5/cmd/registry"
+	cliutil "github.com/rancher/k3d/v5/cmd/util"
+	l "github.com/rancher/k3d/v5/pkg/logger"
+	"github.com/rancher/k3d/v5/pkg/runtimes"
+	"github.com/rancher/k3d/v5/version"
+	"github.com/sirupsen/logrus"
 	"github.com/sirupsen/logrus/hooks/writer"
 )
 
@@ -55,51 +57,26 @@ type RootFlags struct {
 
 var flags = RootFlags{}
 
-// rootCmd represents the base command when called without any subcommands
-var rootCmd = &cobra.Command{
-	Use:   "k3d",
-	Short: "https://k3d.io/ -> Run k3s in Docker!",
-	Long: `https://k3d.io/
+func NewCmdK3d() *cobra.Command {
+
+	// rootCmd represents the base command when called without any subcommands
+	rootCmd := &cobra.Command{
+		Use:   "k3d",
+		Short: "https://k3d.io/ -> Run k3s in Docker!",
+		Long: `https://k3d.io/
 k3d is a wrapper CLI that helps you to easily create k3s clusters inside docker.
 Nodes of a k3d cluster are docker containers running a k3s image.
 All Nodes of a k3d cluster are part of the same docker network.`,
-	Run: func(cmd *cobra.Command, args []string) {
-		if flags.version {
-			printVersion()
-		} else {
-			if err := cmd.Usage(); err != nil {
-				log.Fatalln(err)
+		Run: func(cmd *cobra.Command, args []string) {
+			if flags.version {
+				printVersion()
+			} else {
+				if err := cmd.Usage(); err != nil {
+					l.Log().Fatalln(err)
+				}
 			}
-		}
-	},
-}
-
-// Execute adds all child commands to the root command and sets flags appropriately.
-// This is called by main.main(). It only needs to happen once to the rootCmd.
-func Execute() {
-	if len(os.Args) > 1 {
-		parts := os.Args[1:]
-		// Check if it's a built-in command, else try to execute it as a plugin
-		if _, _, err := rootCmd.Find(parts); err != nil {
-			pluginFound, err := cliutil.HandlePlugin(context.Background(), parts)
-			if err != nil {
-				log.Errorf("Failed to execute plugin '%+v'", parts)
-				log.Fatalln(err)
-			} else if pluginFound {
-				os.Exit(0)
-			}
-		}
+		},
 	}
-	if err := rootCmd.Execute(); err != nil {
-		log.Fatalln(err)
-	}
-}
-
-func GetRootCmd() *cobra.Command {
-	return rootCmd
-}
-
-func init() {
 
 	rootCmd.PersistentFlags().BoolVar(&flags.debugLogging, "verbose", false, "Enable verbose output (debug logging)")
 	rootCmd.PersistentFlags().BoolVar(&flags.traceLogging, "trace", false, "Enable super verbose output (trace logging)")
@@ -109,13 +86,14 @@ func init() {
 	rootCmd.Flags().BoolVar(&flags.version, "version", false, "Show k3d and default k3s version")
 
 	// add subcommands
-	rootCmd.AddCommand(NewCmdCompletion())
+	rootCmd.AddCommand(NewCmdCompletion(rootCmd))
 	rootCmd.AddCommand(cluster.NewCmdCluster())
 	rootCmd.AddCommand(kubeconfig.NewCmdKubeconfig())
 	rootCmd.AddCommand(node.NewCmdNode())
 	rootCmd.AddCommand(image.NewCmdImage())
 	rootCmd.AddCommand(cfg.NewCmdConfig())
 	rootCmd.AddCommand(registry.NewCmdRegistry())
+	rootCmd.AddCommand(debug.NewCmdDebug())
 
 	rootCmd.AddCommand(&cobra.Command{
 		Use:   "version",
@@ -133,11 +111,11 @@ func init() {
 		Run: func(cmd *cobra.Command, args []string) {
 			info, err := runtimes.SelectedRuntime.Info()
 			if err != nil {
-				log.Fatalln(err)
+				l.Log().Fatalln(err)
 			}
 			err = yaml.NewEncoder(os.Stdout).Encode(info)
 			if err != nil {
-				log.Fatalln(err)
+				l.Log().Fatalln(err)
 			}
 		},
 		Hidden: true,
@@ -145,48 +123,72 @@ func init() {
 
 	// Init
 	cobra.OnInitialize(initLogging, initRuntime)
+
+	return rootCmd
+}
+
+// Execute adds all child commands to the root command and sets flags appropriately.
+// This is called by main.main(). It only needs to happen once to the rootCmd.
+func Execute() {
+	cmd := NewCmdK3d()
+	if len(os.Args) > 1 {
+		parts := os.Args[1:]
+		// Check if it's a built-in command, else try to execute it as a plugin
+		if _, _, err := cmd.Find(parts); err != nil {
+			pluginFound, err := cliutil.HandlePlugin(context.Background(), parts)
+			if err != nil {
+				l.Log().Errorf("Failed to execute plugin '%+v'", parts)
+				l.Log().Fatalln(err)
+			} else if pluginFound {
+				os.Exit(0)
+			}
+		}
+	}
+	if err := cmd.Execute(); err != nil {
+		l.Log().Fatalln(err)
+	}
 }
 
 // initLogging initializes the logger
 func initLogging() {
 	if flags.traceLogging {
-		log.SetLevel(log.TraceLevel)
+		l.Log().SetLevel(logrus.TraceLevel)
 	} else if flags.debugLogging {
-		log.SetLevel(log.DebugLevel)
+		l.Log().SetLevel(logrus.DebugLevel)
 	} else {
 		switch logLevel := strings.ToUpper(os.Getenv("LOG_LEVEL")); logLevel {
 		case "TRACE":
-			log.SetLevel(log.TraceLevel)
+			l.Log().SetLevel(logrus.TraceLevel)
 		case "DEBUG":
-			log.SetLevel(log.DebugLevel)
+			l.Log().SetLevel(logrus.DebugLevel)
 		case "WARN":
-			log.SetLevel(log.WarnLevel)
+			l.Log().SetLevel(logrus.WarnLevel)
 		case "ERROR":
-			log.SetLevel(log.ErrorLevel)
+			l.Log().SetLevel(logrus.ErrorLevel)
 		default:
-			log.SetLevel(log.InfoLevel)
+			l.Log().SetLevel(logrus.InfoLevel)
 		}
 	}
-	log.SetOutput(ioutil.Discard)
-	log.AddHook(&writer.Hook{
+	l.Log().SetOutput(ioutil.Discard)
+	l.Log().AddHook(&writer.Hook{
 		Writer: os.Stderr,
-		LogLevels: []log.Level{
-			log.PanicLevel,
-			log.FatalLevel,
-			log.ErrorLevel,
-			log.WarnLevel,
+		LogLevels: []logrus.Level{
+			logrus.PanicLevel,
+			logrus.FatalLevel,
+			logrus.ErrorLevel,
+			logrus.WarnLevel,
 		},
 	})
-	log.AddHook(&writer.Hook{
+	l.Log().AddHook(&writer.Hook{
 		Writer: os.Stdout,
-		LogLevels: []log.Level{
-			log.InfoLevel,
-			log.DebugLevel,
-			log.TraceLevel,
+		LogLevels: []logrus.Level{
+			logrus.InfoLevel,
+			logrus.DebugLevel,
+			logrus.TraceLevel,
 		},
 	})
 
-	formatter := &log.TextFormatter{
+	formatter := &logrus.TextFormatter{
 		ForceColors: true,
 	}
 
@@ -194,18 +196,18 @@ func initLogging() {
 		formatter.FullTimestamp = true
 	}
 
-	log.SetFormatter(formatter)
+	l.Log().SetFormatter(formatter)
 
 }
 
 func initRuntime() {
 	runtime, err := runtimes.GetRuntime("docker")
 	if err != nil {
-		log.Fatalln(err)
+		l.Log().Fatalln(err)
 	}
 	runtimes.SelectedRuntime = runtime
 	if rtinfo, err := runtime.Info(); err == nil {
-		log.Debugf("Runtime Info:\n%+v", rtinfo)
+		l.Log().Debugf("Runtime Info:\n%+v", rtinfo)
 	}
 }
 
@@ -214,43 +216,82 @@ func printVersion() {
 	fmt.Printf("k3s version %s (default)\n", version.K3sVersion)
 }
 
-func generateFishCompletion(writer io.Writer) error {
-	return rootCmd.GenFishCompletion(writer, true)
-}
-
-// Completion
-var completionFunctions = map[string]func(io.Writer) error{
-	"bash": rootCmd.GenBashCompletion,
-	"zsh": func(writer io.Writer) error {
-		if err := rootCmd.GenZshCompletion(writer); err != nil {
-			return err
-		}
-
-		fmt.Fprintf(writer, "\n# source completion file\ncompdef _k3d k3d\n")
-
-		return nil
-	},
-	"psh":        rootCmd.GenPowerShellCompletion,
-	"powershell": rootCmd.GenPowerShellCompletion,
-	"fish":       generateFishCompletion,
-}
-
 // NewCmdCompletion creates a new completion command
-func NewCmdCompletion() *cobra.Command {
+func NewCmdCompletion(rootCmd *cobra.Command) *cobra.Command {
+
+	completionFunctions := map[string]func(io.Writer) error{
+		"bash": rootCmd.GenBashCompletion,
+		"zsh": func(writer io.Writer) error {
+			if err := rootCmd.GenZshCompletion(writer); err != nil {
+				return err
+			}
+
+			fmt.Fprintf(writer, "\n# source completion file\ncompdef _k3d k3d\n")
+
+			return nil
+		},
+		"psh":        rootCmd.GenPowerShellCompletion,
+		"powershell": rootCmd.GenPowerShellCompletionWithDesc,
+		"fish": func(writer io.Writer) error {
+			return rootCmd.GenFishCompletion(writer, true)
+		},
+	}
+
 	// create new cobra command
 	cmd := &cobra.Command{
 		Use:   "completion SHELL",
 		Short: "Generate completion scripts for [bash, zsh, fish, powershell | psh]",
-		Long:  `Generate completion scripts for [bash, zsh, fish, powershell | psh]`,
-		Args:  cobra.ExactArgs(1), // TODO: NewCmdCompletion: add support for 0 args = auto detection
+		Long: `To load completions:
+
+Bash:
+
+	$ source <(k3d completion bash)
+
+	# To load completions for each session, execute once:
+	# Linux:
+	$ k3d completion bash > /etc/bash_completion.d/k3d
+	# macOS:
+	$ k3d completion bash > /usr/local/etc/bash_completion.d/k3d
+
+Zsh:
+
+	# If shell completion is not already enabled in your environment,
+	# you will need to enable it.  You can execute the following once:
+
+	$ echo "autoload -U compinit; compinit" >> ~/.zshrc
+
+	# To load completions for each session, execute once:
+	$ k3d completion zsh > "${fpath[1]}/k3d"
+
+	# You will need to start a new shell for this setup to take effect.
+
+fish:
+
+	$ k3d completion fish | source
+
+	# To load completions for each session, execute once:
+	$ k3d completion fish > ~/.config/fish/completions/k3d.fish
+
+PowerShell:
+
+	PS> k3d completion powershell | Out-String | Invoke-Expression
+
+	# To load completions for every new session, run:
+	PS> k3d completion powershell > k3d.ps1
+	# and source this file from your PowerShell profile.
+`,
+		ValidArgs:             []string{"bash", "zsh", "fish", "powershell"},
+		ArgAliases:            []string{"psh"},
+		DisableFlagsInUseLine: true,
+		Args:                  cobra.ExactValidArgs(1),
 		Run: func(cmd *cobra.Command, args []string) {
 			if completionFunc, ok := completionFunctions[args[0]]; ok {
 				if err := completionFunc(os.Stdout); err != nil {
-					log.Fatalf("Failed to generate completion script for shell '%s'", args[0])
+					l.Log().Fatalf("Failed to generate completion script for shell '%s'", args[0])
 				}
 				return
 			}
-			log.Fatalf("Shell '%s' not supported for completion", args[0])
+			l.Log().Fatalf("Shell '%s' not supported for completion", args[0])
 		},
 	}
 	return cmd
