@@ -23,8 +23,8 @@ THE SOFTWARE.
 package config
 
 import (
-	conf "github.com/rancher/k3d/v4/pkg/config/v1alpha2"
-	log "github.com/sirupsen/logrus"
+	conf "github.com/rancher/k3d/v5/pkg/config/v1alpha3"
+	l "github.com/rancher/k3d/v5/pkg/logger"
 )
 
 // ProcessClusterConfig applies processing to the config sanitizing it and doing
@@ -32,15 +32,12 @@ import (
 func ProcessClusterConfig(clusterConfig conf.ClusterConfig) (*conf.ClusterConfig, error) {
 	cluster := clusterConfig.Cluster
 	if cluster.Network.Name == "host" {
-		log.Infoln("Hostnetwork selected - disabling injection of docker host into the cluster, server load balancer and setting the api port to the k3s default")
+		l.Log().Infoln("Hostnetwork selected - disabling injection of docker host into the cluster, server load balancer and setting the api port to the k3s default")
 		// if network is set to host, exposed api port must be the one imposed by k3s
 		k3sPort := cluster.KubeAPI.Port.Port()
-		log.Debugf("Host network was chosen, changing provided/random api port to k3s:%s", k3sPort)
+		l.Log().Debugf("Host network was chosen, changing provided/random api port to k3s:%s", k3sPort)
 		cluster.KubeAPI.PortMapping.Binding.HostPort = k3sPort
 
-		// if network is host, dont inject docker host into the cluster
-		clusterConfig.ClusterCreateOpts.PrepDisableHostIPInjection = true
-
 		// if network is host, disable load balancer
 		// serverlb not supported in hostnetwork mode due to port collisions with server node
 		clusterConfig.ClusterCreateOpts.DisableLoadBalancer = true
