@@ -26,7 +26,8 @@ import (
 	"context"
 	"testing"
 
-	"github.com/rancher/k3d/v4/pkg/runtimes"
+	conf "github.com/rancher/k3d/v5/pkg/config/v1alpha3"
+	"github.com/rancher/k3d/v5/pkg/runtimes"
 	"github.com/spf13/viper"
 	"gotest.tools/assert"
 )
@@ -38,14 +39,14 @@ func TestProcessClusterConfig(t *testing.T) {
 	vip.SetConfigFile(cfgFile)
 	_ = vip.ReadInConfig()
 
-	cfg, err := FromViperSimple(vip)
+	cfg, err := FromViper(vip)
 	if err != nil {
 		t.Error(err)
 	}
 
 	t.Logf("\n========== Read Config and transform to cluster ==========\n%+v\n=================================\n", cfg)
 
-	clusterCfg, err := TransformSimpleToClusterConfig(context.Background(), runtimes.Docker, cfg)
+	clusterCfg, err := TransformSimpleToClusterConfig(context.Background(), runtimes.Docker, cfg.(conf.SimpleConfig))
 	if err != nil {
 		t.Error(err)
 	}
@@ -54,7 +55,6 @@ func TestProcessClusterConfig(t *testing.T) {
 
 	clusterCfg, err = ProcessClusterConfig(*clusterCfg)
 	assert.Assert(t, clusterCfg.ClusterCreateOpts.DisableLoadBalancer == false, "The load balancer should be enabled")
-	assert.Assert(t, clusterCfg.ClusterCreateOpts.PrepDisableHostIPInjection == false, "The host ip injection should be enabled")
 
 	t.Logf("\n===== Resulting Cluster Config (non-host network) =====\n%+v\n===============\n", clusterCfg)
 
@@ -63,7 +63,6 @@ func TestProcessClusterConfig(t *testing.T) {
 	clusterCfg.Cluster.Network.Name = "host"
 	clusterCfg, err = ProcessClusterConfig(*clusterCfg)
 	assert.Assert(t, clusterCfg.ClusterCreateOpts.DisableLoadBalancer == true, "The load balancer should be disabled")
-	assert.Assert(t, clusterCfg.ClusterCreateOpts.PrepDisableHostIPInjection == true, "The host ip injection should be disabled")
 
 	t.Logf("\n===== Resulting Cluster Config (host network) =====\n%+v\n===============\n", clusterCfg)
 
