@@ -33,20 +33,20 @@ import (
 	"github.com/docker/docker/api/types/container"
 	"github.com/docker/docker/api/types/filters"
 	"github.com/docker/docker/client"
-	k3d "github.com/rancher/k3d/v4/pkg/types"
-	log "github.com/sirupsen/logrus"
+	l "github.com/rancher/k3d/v5/pkg/logger"
+	k3d "github.com/rancher/k3d/v5/pkg/types"
+	"github.com/sirupsen/logrus"
 )
 
 // createContainer creates a new docker container from translated specs
 func createContainer(ctx context.Context, dockerNode *NodeInDocker, name string) (string, error) {
 
-	log.Tracef("Creating docker container with translated config\n%+v\n", dockerNode)
+	l.Log().Tracef("Creating docker container with translated config\n%+v\n", dockerNode)
 
 	// initialize docker client
 	docker, err := GetDockerClient()
 	if err != nil {
-		log.Errorln("Failed to create docker client")
-		return "", err
+		return "", fmt.Errorf("failed to create docker client: %w", err)
 	}
 	defer docker.Close()
 
@@ -57,15 +57,13 @@ func createContainer(ctx context.Context, dockerNode *NodeInDocker, name string)
 		if err != nil {
 			if client.IsErrNotFound(err) {
 				if err := pullImage(ctx, docker, dockerNode.ContainerConfig.Image); err != nil {
-					log.Errorf("Failed to create container '%s'", name)
-					return "", err
+					return "", fmt.Errorf("docker failed to pull image '%s': %w", dockerNode.ContainerConfig.Image, err)
 				}
 				continue
 			}
-			log.Errorf("Failed to create container '%s'", name)
-			return "", err
+			return "", fmt.Errorf("docker failed to create container '%s': %w", name, err)
 		}
-		log.Debugf("Created container %s (ID: %s)", name, resp.ID)
+		l.Log().Debugf("Created container %s (ID: %s)", name, resp.ID)
 		break
 	}
 
@@ -76,8 +74,7 @@ func startContainer(ctx context.Context, ID string) error {
 	// initialize docker client
 	docker, err := GetDockerClient()
 	if err != nil {
-		log.Errorln("Failed to create docker client")
-		return err
+		return fmt.Errorf("failed to get docker client: %w", err)
 	}
 	defer docker.Close()
 
@@ -90,8 +87,7 @@ func removeContainer(ctx context.Context, ID string) error {
 	// (0) create docker client
 	docker, err := GetDockerClient()
 	if err != nil {
-		log.Errorln("Failed to create docker client")
-		return err
+		return fmt.Errorf("failed to get docker client: %w", err)
 	}
 	defer docker.Close()
 
@@ -103,11 +99,10 @@ func removeContainer(ctx context.Context, ID string) error {
 
 	// (2) remove container
 	if err := docker.ContainerRemove(ctx, ID, options); err != nil {
-		log.Errorf("Failed to delete container '%s'", ID)
-		return err
+		return fmt.Errorf("docker failed to remove the container '%s': %w", ID, err)
 	}
 
-	log.Infoln("Deleted", ID)
+	l.Log().Infoln("Deleted", ID)
 
 	return nil
 }
@@ -117,22 +112,20 @@ func pullImage(ctx context.Context, docker *client.Client, image string) error {
 
 	resp, err := docker.ImagePull(ctx, image, types.ImagePullOptions{})
 	if err != nil {
-		log.Errorf("Failed to pull image '%s'", image)
-		return err
+		return fmt.Errorf("docker failed to pull the image '%s': %w", image, err)
 	}
 	defer resp.Close()
 
-	log.Infof("Pulling image '%s'", image)
+	l.Log().Infof("Pulling image '%s'", image)
 
 	// in debug mode (--verbose flag set), output pull progress
 	var writer io.Writer = ioutil.Discard
-	if log.GetLevel() == log.DebugLevel {
+	if l.Log().GetLevel() == logrus.DebugLevel {
 		writer = os.Stdout
 	}
 	_, err = io.Copy(writer, resp)
 	if err != nil {
-		log.Warningf("Couldn't get docker output")
-		log.Warningln(err)
+		l.Log().Warnf("Couldn't get docker output: %v", err)
 	}
 
 	return nil
@@ -144,14 +137,13 @@ func getNodeContainer(ctx context.Context, node *k3d.Node) (*types.Container, er
 	// (0) create docker client
 	docker, err := GetDockerClient()
 	if err != nil {
-		log.Errorln("Failed to create docker client")
-		return nil, err
+		return nil, fmt.Errorf("failed to get docker client: %w", err)
 	}
 	defer docker.Close()
 
 	// (1) list containers which have the default k3d labels attached
 	filters := filters.NewArgs()
-	for k, v := range node.Labels {
+	for k, v := range node.RuntimeLabels {
 		filters.Add("label", fmt.Sprintf("%s=%s", k, v))
 	}
 
@@ -186,8 +178,7 @@ func getNodeContainer(ctx context.Context, node *k3d.Node) (*types.Container, er
 func executeCheckInContainer(ctx context.Context, image string, cmd []string) (int64, error) {
 	docker, err := GetDockerClient()
 	if err != nil {
-		log.Errorln("Failed to create docker client")
-		return -1, err
+		return -1, fmt.Errorf("failed to create docker client: %w", err)
 	}
 	defer docker.Close()
 
@@ -203,20 +194,17 @@ func executeCheckInContainer(ctx context.Context, image string, cmd []string) (i
 		if err != nil {
 			if client.IsErrNotFound(err) {
 				if err := pullImage(ctx, docker, image); err != nil {
-					log.Errorf("Failed to create container from image %s with cmd %s", image, cmd)
-					return -1, err
+					return -1, fmt.Errorf("docker failed to pull image '%s': %w", image, err)
 				}
 				continue
 			}
-			log.Errorf("Failed to create container from image %s with cmd %s", image, cmd)
-			return -1, err
+			return -1, fmt.Errorf("docker failed to create container from image '%s' with cmd '%s': %w", image, cmd, err)
 		}
 		break
 	}
 
 	if err = startContainer(ctx, resp.ID); err != nil {
-		log.Errorf("Failed to start container from image %s with cmd %s", image, cmd)
-		return -1, err
+		return -1, fmt.Errorf("docker failed to start container from image '%s' with cmd '%s': %w", image, cmd, err)
 	}
 
 	exitCode := -1
@@ -224,15 +212,14 @@ func executeCheckInContainer(ctx context.Context, image string, cmd []string) (i
 	select {
 	case err := <-errCh:
 		if err != nil {
-			log.Errorf("Error while waiting for container %s to exit", resp.ID)
-			return -1, err
+			return -1, fmt.Errorf("docker error while waiting for container '%s' to exit: %w", resp.ID, err)
 		}
 	case status := <-statusCh:
 		exitCode = int(status.StatusCode)
 	}
 
 	if err = removeContainer(ctx, resp.ID); err != nil {
-		return -1, err
+		return -1, fmt.Errorf("docker failed to remove container '%s': %w", resp.ID, err)
 	}
 
 	return int64(exitCode), nil
@@ -240,11 +227,10 @@ func executeCheckInContainer(ctx context.Context, image string, cmd []string) (i
 
 // CheckIfDirectoryExists checks for the existence of a given path inside the docker environment
 func CheckIfDirectoryExists(ctx context.Context, image string, dir string) (bool, error) {
-	log.Tracef("checking if dir %s exists in docker environment...", dir)
+	l.Log().Tracef("checking if dir %s exists in docker environment...", dir)
 	shellCmd := fmt.Sprintf("[ -d \"%s\" ] && exit 0 || exit 1", dir)
 	cmd := []string{"sh", "-c", shellCmd}
 	exitCode, err := executeCheckInContainer(ctx, image, cmd)
-	log.Tracef("check dir container returned %d exist code", exitCode)
+	l.Log().Tracef("check dir container returned %d exit code", exitCode)
 	return exitCode == 0, err
-
 }
